@import 'theme.css';
@import url('https://fonts.googleapis.com/css2?family=Overpass+Mono:wght@300;400&display=swap');

/* Some of our section titles are looong */
.wy-menu-vertical { width: 340px; }

/* Many of our diagnostics are even longer */
.rst-content pre.literal-block, .rst-content div[class^="highlight"] pre, .rst-content .linenodiv pre { white-space: pre-wrap; }

div.wy-nav-content {
    padding: 1.618em 3.236em;
    height: 100%;
    max-width: 1000px;
    margin: auto;
}

div.highlight {
    border-radius: 5px;
}

div.admonition {
    border-radius: 0px 0px 5px 5px;
}

p.admonition-title {
    border-radius: 5px 5px 0px 0px;
}

code.literal {
    border-radius: 5px;
}

svg.WaveDrom {
    padding: 10px;
    border-radius: 5px;
    background: #f0f0f0;
    border: 1px solid #ccc;
    margin: 10px;
}

svg.WaveDrom text {
    font-size: 11pt;
    font-style: normal;
    font-variant: normal;
    font-weight: 400;
    font-stretch: normal;
    text-align: center;
    fill-opacity: 1;
    font-family: 'Overpass Mono', monospace
}

svg.WaveDrom .h1 {
    font-size: 33pt;
    font-weight: 700
}

svg.WaveDrom .h2 {
    font-size: 27pt;
    font-weight: 700
}

svg.WaveDrom .h3 {
    font-size: 20pt;
    font-weight: 700
}

svg.WaveDrom .h4 {
    font-size: 14pt;
    font-weight: 700
}

svg.WaveDrom .h5 {
    font-size: 11pt;
    font-weight: 700
}

svg.WaveDrom .h6 {
    font-size: 8pt;
    font-weight: 700
}

svg.WaveDrom .muted {
    fill: #D4D4D4
}

svg.WaveDrom .warning {
    fill: #F7E740
}

svg.WaveDrom .error {
    fill: #CD427E
}

svg.WaveDrom .info {
    fill: #53A9FD
}

svg.WaveDrom .success {
    fill: #59E56D
}

svg.WaveDrom .s1 {
    fill: none;
    stroke: #303136;
    stroke-width: 1;
    stroke-linecap: round;
    stroke-linejoin: miter;
    stroke-miterlimit: 4;
    stroke-opacity: 1;
    stroke-dasharray: none
}

svg.WaveDrom .s2 {
    fill: none;
    stroke: #303136;
    stroke-width: .5;
    stroke-linecap: round;
    stroke-linejoin: miter;
    stroke-miterlimit: 4;
    stroke-opacity: 1;
    stroke-dasharray: none
}

svg.WaveDrom .s3 {
    color: #303136;
    fill: none;
    stroke: #303136;
    stroke-width: 1;
    stroke-linecap: round;
    stroke-linejoin: miter;
    stroke-miterlimit: 4;
    stroke-opacity: 1;
    stroke-dasharray: 1, 3;
    stroke-dashoffset: 0;
    marker: none;
    visibility: visible;
    display: inline;
    overflow: visible
}

svg.WaveDrom .s4 {
    color: #303136;
    fill: none;
    stroke: #303136;
    stroke-width: 1;
    stroke-linecap: round;
    stroke-linejoin: miter;
    stroke-miterlimit: 4;
    stroke-opacity: 1;
    stroke-dasharray: none;
    stroke-dashoffset: 0;
    marker: none;
    visibility: visible;
    display: inline;
    overflow: visible
}

svg.WaveDrom .s5 {
    fill: #FDFDFD;
    stroke: none
}

svg.WaveDrom .s6 {
    fill: #303136;
    fill-opacity: 1;
    stroke: none
}

svg.WaveDrom .s7 {
    color: #303136;
    fill: #FDFDFD;
    fill-opacity: 1;
    fill-rule: nonzero;
    stroke: none;
    stroke-width: 1px;
    marker: none;
    visibility: visible;
    display: inline;
    overflow: visible
}

svg.WaveDrom .s8 {
    color: #303136;
    fill: #F1994B;
    fill-opacity: 1;
    fill-rule: nonzero;
    stroke: none;
    stroke-width: 1px;
    marker: none;
    visibility: visible;
    display: inline;
    overflow: visible
}

svg.WaveDrom .s9 {
    color: #303136;
    fill: #26A3D9;
    fill-opacity: 1;
    fill-rule: nonzero;
    stroke: none;
    stroke-width: 1px;
    marker: none;
    visibility: visible;
    display: inline;
    overflow: visible
}

svg.WaveDrom .s10 {
    color: #303136;
    fill: #AF92FB;
    fill-opacity: 1;
    fill-rule: nonzero;
    stroke: none;
    stroke-width: 1px;
    marker: none;
    visibility: visible;
    display: inline;
    overflow: visible
}

svg.WaveDrom .s11 {
    color: #303136;
    fill: #53A9FD;
    fill-opacity: 1;
    fill-rule: nonzero;
    stroke: none;
    stroke-width: 1px;
    marker: none;
    visibility: visible;
    display: inline;
    overflow: visible
}

svg.WaveDrom .s12 {
    color: #303136;
    fill: #00994D;
    fill-opacity: 1;
    fill-rule: nonzero;
    stroke: none;
    stroke-width: 1px;
    marker: none;
    visibility: visible;
    display: inline;
    overflow: visible
}

svg.WaveDrom .s13 {
    color: #303136;
    fill: #FF6D7F;
    fill-opacity: 1;
    fill-rule: nonzero;
    stroke: none;
    stroke-width: 1px;
    marker: none;
    visibility: visible;
    display: inline;
    overflow: visible
}

svg.WaveDrom .s14 {
    color: #303136;
    fill: #F1994B;
    fill-opacity: 1;
    fill-rule: nonzero;
    stroke: none;
    stroke-width: 1px;
    marker: none;
    visibility: visible;
    display: inline;
    overflow: visible
}

svg.WaveDrom .s15 {
    fill: #53A9FD;
    fill-opacity: 1;
    stroke: none
}

svg.WaveDrom .s16 {
    fill: none;
    stroke: #53A9FD;
    stroke-width: 1;
    stroke-linecap: round;
    stroke-linejoin: miter;
    stroke-miterlimit: 4;
    stroke-opacity: 1;
    stroke-dasharray: none
}
